#include <stdio.h> // Needed for the print function. 
#include <windows.h>

#include "RingRingPythonCalling.h" // We need to include our header as else it does not know how the class is build up. 
 

namespace PythonExtensions {
	
	/*

	Here we put all our class functions and normal c++ functions. 
	Python will not be able to call these functions directly. 
	My guess is there is a way but I have not found it yet. 

	See the C wrapper how we actually solve this problem.
	*/


	PythonCalls::PythonCalls( )
		: rows(0), columns(0),valid(true)// We initialize our rows and columns to 0.
		// This is the constructor. Its like the __init__ in a python class. 
	{
		// We set the valid bool to true so we can check from python if the object still lives in memory. 
		

		
		
	}

	PythonCalls::~PythonCalls( )
		// This is the deconstructor. It gets called when the object is deleted from memory. 
	{
		// Here we set our boolean to false so we can check from python if we can still use the class. 
		valid = false;

		// In here we should also delete any other objects i.e. our array. 
		// But I'll leave that for now. 
	}

	int PythonCalls::CreateArray( double ** pythonArray,int rowsSize, int columnsSize )
	{
		// Creates an array from the python data. 
		rows = (int)rowsSize;
		columns = (int)columnsSize;
		// First we create an empty array with the correct size. 
		// We could skip this step and it would still work in this case ...
		// However then we would most likely be overwriting some chunk of memory somewhere 
		// which is bad m'kay :P
		CreateArray(rows,columns); 

		// And now we can assing our python array to our array member. 
		array2D = pythonArray;
		return 1;
	};
	int PythonCalls::CreateArray(int rows, int columns)
	{
		// Creates an empty array with the set size. 
		array2D = new double *[rows];

		for (int y=0;y<rows;y++)
		{
			array2D[y] = new double[columns];
			for (int x=0;x<columns;x++)
			{
				array2D[y][x] = 0;
			}
		}
		return 1;
	}

	int PythonCalls::PrintArray( )
	{
		printf("\nPrinting from c++. \n");
		// So printing it we just itterate over the rows and columns. 
		for (int y=0;y<rows;y++)
		{
			for (int x=0;x<columns;x++)
			{
				printf("\t%.3f \t", array2D[y][x]);
			}
			printf("\n");
		}
		printf("End c++ print\n\n");
		return 1;
	}

	int PythonCalls::SmoothArray(int percentage)
	{
		// A simple smoothing function. 
		// Im sure there are 1000 ways to do this better but you get the idea. 
		double total = 0;
		double mean = 0;
		double perc = (double)percentage *0.01;
		//printf("rows %d, columns %d",rows,columns );
		for (int y=0;y<rows;y++)
		{
			total = 0;
			for (int x=0;x<columns;x++)
			{
				total += array2D[y][x];
			}
			mean = total / (double)columns;
			for (int x=0;x<columns;x++)
			{
				if (array2D[y][x] > mean)
				{
					array2D[y][x] = array2D[y][x]- ((array2D[y][x]-mean)*perc);
				}
				else if (array2D[y][x] < mean)
				{
					array2D[y][x] = array2D[y][x] + ((mean - array2D[y][x])*perc);
				}
			}
		}
		Sleep((DWORD)0.00001); // Required for python.
		return 1;

	}
	bool PythonCalls::IsValid()
	{
		// This function is not really correct as if the object gets deleted so does this function. 
		// But python will still hold a version of it and thus be able to call it and know that its no longer valid. 
		return valid;
	}
	double ** PythonCalls::GetArray()
	{
		return array2D;
	}

	double ** PythonCalls::GetArrayCopy()
	{
		double ** arrayCopy =  new double *[rows];
		for (int y=0;y<rows;y++)
		{
			arrayCopy[y] = new double[columns];
			for (int x=0;x<columns;x++)
			{
				arrayCopy[y][x] = array2D[y][x];
			}
		}
		return arrayCopy;
	}
	//////////////////////////////////////////////////////////////////////////
	// C Wrapper
	//////////////////////////////////////////////////////////////////////////
	// Wrapper in C so python can access the functions 
	extern "C"
	{
		__declspec(dllexport)
		PythonExtensions::PythonCalls * PythonCallsNew() {
			return new PythonExtensions::PythonCalls();
		}

		__declspec(dllexport)
			int create_array(PythonExtensions::PythonCalls * pythonCallsClass, double ** pythonArray, int rows, int columns )
		{
			return pythonCallsClass->CreateArray(pythonArray, rows, columns);
		}

		__declspec(dllexport)
			int create_empty_array(PythonExtensions::PythonCalls * pythonCallsClass, int rows, int columns )
		{
			return pythonCallsClass->CreateArray( rows, columns);
		}

		__declspec(dllexport)
			int print_array(PythonExtensions::PythonCalls * pythonCallsClass)
		{
			return pythonCallsClass->PrintArray();
		}

		__declspec(dllexport)
			int smooth_array(PythonExtensions::PythonCalls * pythonCallsClass, int percentage)
		{
			return pythonCallsClass->SmoothArray(percentage);
		}

		__declspec(dllexport)
			bool is_valid(PythonExtensions::PythonCalls * pythonCallsClass)
		{
			return pythonCallsClass->IsValid();
		}

		__declspec(dllexport)
			double ** get_array(PythonExtensions::PythonCalls * pythonCallsClass)
		{
			return pythonCallsClass->GetArray();
		}

		__declspec(dllexport)
			double ** get_array_copy(PythonExtensions::PythonCalls * pythonCallsClass)
		{
			return pythonCallsClass->GetArrayCopy();
		}
	}

}